package userRules;

import cellularAutomata.rules.templates.IntegerRuleTemplate;

import javax.swing.*;
import java.util.Random;

/**
 * Created by ano on 14.07.16.
 */
public class VsRule extends IntegerRuleTemplate {

    private static final String RULE_NAME = "PvP Rule";
    private static final String TOOLTIP =
            "<html><body><b>"+RULE_NAME +".</b> 1vs1 rule."
                    + "<body></html>";
    Random random = new Random();

    //Initialize states
    private final int notUser=0;
    private final int new1=1, old1=2;
    private final int new2=11, old2=12;
    private final int dead1=21, dead2=22;

    //Initialize sums
    private static int sum1,sum2;

    //Initialize probabilities
    private static double probBirth1, probBirth2;
    private static double probDie1, probDie2;
    private static double probReturn1, probReturn2;


    //Initialize thresholds
    private static int birthThreshold1, birthThreshold2;
    private static int deathThreshold1, deathThreshold2; //Die if too much
    private static int attackThreshold1, attackThreshold2;

    static volatile JSpinner birthThresholdSpin1, birthThresholdSpin2;
    static volatile JSpinner deathThresholdSpin1, deathThresholdSpin2;
    static volatile JSpinner probBirthSpin1, probBirthSpin2;
    static volatile JSpinner probDieSpin1, probDieSpin2;
    static volatile JSpinner attackThresholdSpin1, attackThresholdSpin2;
    static volatile JSpinner probReturnSpin1, probReturnSpin2;

    public static double random(double min, double max) {
        return (new Random().nextDouble() * (max - min)) + min;
    }







    @Override
    protected int integerRule(int cell, int[] neighbors, int numStates, int generation) {
        sum1=0;sum2=0;
        birthThreshold1=((SpinnerNumberModel)birthThresholdSpin1.getModel()).getNumber().intValue();
        birthThreshold2=((SpinnerNumberModel)birthThresholdSpin2.getModel()).getNumber().intValue();
        deathThreshold1=((SpinnerNumberModel)deathThresholdSpin1.getModel()).getNumber().intValue();
        deathThreshold2=((SpinnerNumberModel)deathThresholdSpin2.getModel()).getNumber().intValue();
        attackThreshold1=((SpinnerNumberModel)attackThresholdSpin1.getModel()).getNumber().intValue();
        attackThreshold2=((SpinnerNumberModel)attackThresholdSpin2.getModel()).getNumber().intValue();
        probBirth1=((SpinnerNumberModel)probBirthSpin1.getModel()).getNumber().doubleValue();
        probBirth2=((SpinnerNumberModel)probBirthSpin2.getModel()).getNumber().doubleValue();
        probDie1=((SpinnerNumberModel)probDieSpin1.getModel()).getNumber().doubleValue();
        probDie2=((SpinnerNumberModel)probDieSpin2.getModel()).getNumber().doubleValue();
        probReturn1=((SpinnerNumberModel)probReturnSpin1.getModel()).getNumber().doubleValue();
        probReturn2=((SpinnerNumberModel)probReturnSpin2.getModel()).getNumber().doubleValue();


        for (int i = 0; i < neighbors.length; i++) {
                if (neighbors[i] == new1||neighbors[i] == old1) sum1++;
                if (neighbors[i] == new2||neighbors[i] == old2) sum2++;
        }

        //Rules for new
        if (cell==new1) return old1;
        if (cell==new2) return old2;
        //Rule for Null
        if (cell==notUser) {
            if ((sum1*probBirth1>sum2*probBirth2)&&(sum1>birthThreshold1)&&(random(0, 100)<(sum1*probBirth1)))
                return new1;
            if ((sum1*probBirth1<sum2*probBirth2)&&(sum2>birthThreshold2)&&(random(0, 100)<(sum2*probBirth2)))
                return new2;
            if (sum1*probBirth1==sum2*probBirth2) {
                if ((random.nextInt(2) + 1 == 1) && (sum1 > birthThreshold1) && (random(0, 100) < (sum1 * probBirth1)))
                    return new1;
                if ((random.nextInt(2) + 1 == 2)&&(sum2>birthThreshold2)&&(random(0, 100)<(sum2*probBirth2)))
                    return new2;
            }

        }
        //Rules for classic
        if (cell==old1) {
            //if ((sum1*probBirth1>=sum2*probBirth2)&&(sum1<deathThreshold1)) return old1;
            if (sum1>=deathThreshold1) {
                if (random(0, 100)<probDie1*1) return dead1;
            }
            if ((sum2*probBirth2>(sum1+1)*probBirth1)&&(sum2>=attackThreshold2)&&(random(0, 100)<(sum2*probBirth2))) return old2;
        }
        if (cell==old2) {
            //if ((sum2*probBirth2>=sum1*probBirth1)&&(sum2<deathThreshold2)) return old2;
            if (sum2>=deathThreshold2) {
                if (random(0, 100)<probDie2) return dead2;
            }
            if ((sum1*probBirth1>(sum2+1)*probBirth2)&&(sum1>=attackThreshold1)&&(random(0, 100)<(sum1*probBirth1))) return old1;
        }
        //Rules for dead
        if (cell==dead1||cell==dead2) {
            if ((sum1*probReturn1>sum2*probReturn2)&&(random(0, 100)<(sum1*probReturn1))) return old1;
            if ((sum1*probReturn1<sum2*probReturn2)&&(random(0, 100)<(sum2*probReturn2))) return old2;
        }

        return cell;
    }

    public VsRule (boolean minimalOrLazyInitialization)
    {
        super(minimalOrLazyInitialization);
    }

    public String getDisplayName()
    {
        return RULE_NAME;
    }
    /**
     * A brief description (written in HTML) that describes this rule. The
     * description will be displayed as a tooltip. Using html permits line
     * breaks, font colors, etcetera, as described in HTML resources. Regular
     * line breaks will not work.
     *
     * @return An HTML string describing this rule.
     */
    public String getToolTipDescription()
    {
        return TOOLTIP;
    }

    public JPanel getAdditionalPropertiesPanel(){
        JPanel jPanel = new JPanel();
        jPanel.setLayout(new BoxLayout(jPanel,BoxLayout.PAGE_AXIS));
        jPanel.add(new JLabel("Минимальное кол-во соседей"));
        jPanel.add(new JLabel("для рождения клетки"));
        jPanel.add(new JLabel("1:"));
        birthThresholdSpin1 = new JSpinner(new SpinnerNumberModel(1,0,24,1));
        jPanel.add(birthThresholdSpin1);
        jPanel.add(new JLabel("2:"));
        birthThresholdSpin2 = new JSpinner(new SpinnerNumberModel(1,0,24,1));
        jPanel.add(birthThresholdSpin2);
        JLabel birthLbl = new JLabel("Влияние пользователей на клетку");
        jPanel.add(birthLbl);
        jPanel.add(new JLabel("(Вероятность рассчитывается как кол-во"));
        jPanel.add(new JLabel("живых клеток * влияние)"));
        jPanel.add(new JLabel("1:"));
        probBirthSpin1 = new JSpinner(new SpinnerNumberModel(1,0,100,0.001));
        jPanel.add(probBirthSpin1);
        jPanel.add(new JLabel("2:"));
        probBirthSpin2 = new JSpinner(new SpinnerNumberModel(1,0,100,0.001));
        jPanel.add(probBirthSpin2);
        jPanel.add(new JLabel("Минимальное кол-во соседей"));
        jPanel.add(new JLabel("для перехвата клетки"));
        jPanel.add(new JLabel("1:"));
        attackThresholdSpin1 = new JSpinner(new SpinnerNumberModel(10,0,24,1));
        jPanel.add(attackThresholdSpin1);
        jPanel.add(new JLabel("2:"));
        attackThresholdSpin2 = new JSpinner(new SpinnerNumberModel(10,0,24,1));
        jPanel.add(attackThresholdSpin2);
        jPanel.add(new JLabel("Минимальное кол-во соседей"));
        jPanel.add(new JLabel("для отказа от перенаселения"));
        jPanel.add(new JLabel("1:"));
        deathThresholdSpin1 = new JSpinner(new SpinnerNumberModel(23,0,24,1));
        jPanel.add(deathThresholdSpin1);
        jPanel.add(new JLabel("2:"));
        deathThresholdSpin2 = new JSpinner(new SpinnerNumberModel(23,0,24,1));
        jPanel.add(deathThresholdSpin2);
        JLabel tooMuchDeathLbl = new JLabel("Вероятность отказа от перенеаселения");
        jPanel.add(tooMuchDeathLbl);
        jPanel.add(new JLabel("(0.001-100)"));
        jPanel.add(new JLabel("1:"));
        probDieSpin1= new JSpinner(new SpinnerNumberModel(1,0,100,0.001));
        jPanel.add(probDieSpin1);
        jPanel.add(new JLabel("2:"));
        probDieSpin2= new JSpinner(new SpinnerNumberModel(1,0,100,0.001));
        jPanel.add(probDieSpin2);
        JLabel returnLbl = new JLabel("Вероятность возвращения после отказа");
        jPanel.add(returnLbl);
        jPanel.add(new JLabel("1:"));
        probReturnSpin1 = new JSpinner(new SpinnerNumberModel(1,0,100,0.001));
        jPanel.add(probReturnSpin1);
        jPanel.add(new JLabel("2:"));
        probReturnSpin2 = new JSpinner(new SpinnerNumberModel(1,0,100,0.001));
        jPanel.add(probReturnSpin2);


        return jPanel;
    }
}
